import gettext
import re
import time
import gi
import os
import platform
import subprocess
import locale
import threading

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio, Gdk, GdkPixbuf, GLib, GObject

gettext.install("redoswelcome", "/usr/share/locale")
# gettext.install("redoswelcome", "data/locale")

from locale import gettext as _
from pydbus import SystemBus

locale.bindtextdomain("redoswelcome", "/usr/share/locale")
# locale.bindtextdomain("redoswelcome", "data/locale")

locale.textdomain("redoswelcome")

LAYOUT_STYLE_LEGACY, LAYOUT_STYLE_NEW = range(2)



bus = SystemBus()
the_dbus_starter = bus.get("org.redoswelcome.pydbus.Starter")
whoami = GLib.get_user_name()

from redoswelcome.common import AsyncWorker
# from common import AsyncWorker

class MyException(Exception):
    pass

class RedosWelcomeDNF:
    def __init__(self, rpm_path, utility_label, label_start_text, utility_button):
        self.rpm = rpm_path
        builder = Gtk.Builder()
        builder.set_translation_domain("redoswelcome")

        builder.add_from_file('/usr/share/redos/redoswelcome/ui/templates.glade')
        # builder.add_from_file('data/ui/templates.glade')

        self.rpm = " ".join(rpm for rpm in rpm_path)
        self.utility_label = utility_label
        self.label_start_text = label_start_text
        self.button_utility = utility_button
        self.window = builder.get_object("rpminstall")
        self.window.set_title(_("Rpm install"))

        self.rpm_install_stack = builder.get_object("rpm_install_stack")
        self.rpm_info_notebook = builder.get_object("rpm_info_notebook")
        self.error_rpm_textview = builder.get_object("error_rpm_textview")

        self.resolve_dep_rpm_local = builder.get_object("resolve_dep_rpm_local")
        self.resolve_deps_local()
        self.deny_transaction_rpm = builder.get_object("deny_transaction_rpm")
        self.close_rpm_error = builder.get_object("close_rpm_error")
        self.close_rpm_error.connect("clicked", self.close)

        self.close_rpm_success = builder.get_object("close_rpm_success")
        self.close_rpm_success.connect("clicked", self.close)

        self.deny_transaction_rpm.connect("clicked", self.close)
        self.accept_transaction_rpm = builder.get_object("accept_transaction_rpm")
        self.accept_transaction_rpm.connect("clicked", self.on_accept_transaction_rpm_clicked)
        self.window.show()
        self.progressbar = builder.get_object("progressbar")
        self.timeout_id = GLib.timeout_add(60, self.on_timeout, None)

    def on_timeout(self, user_data):
        self.progressbar.pulse()
        return True

    def resolve_deps_local(self):
        out = os.popen("systemctl is-active redoswelcome-dbus.service").readline().strip()
        if out == "inactive":
            the_dbus_starter.start()
        self.redos_dnf = bus.get("org.redoswelcome.pydbus.RedosWelcomeDNF", timeout=600)
        if "r7-release" in self.rpm and len(self.rpm) > len("r7-release"):
            result = self.redos_dnf.dnf_command_gpg("r7-release", "/etc/pki/rpm-gpg/RPM-GPG-KEY-R7-OFFICE.public")
            self.rpm = re.sub("r7-release", "", self.rpm)
            if result == "0":
                self.resolve_extra_func()
            else:
                self.rpm_install_stack.set_visible_child_name("error_rpm_textview_page")
                self.error_rpm_textview.set_text(result)
        elif "yandex-browser-release" in self.rpm and len(self.rpm) > len("yandex-browser-release"):
            result = self.redos_dnf.dnf_command_gpg("yandex-browser-release", "")
            self.rpm = re.sub("yandex-browser-release", "", self.rpm)
            if result == "0":
                self.resolve_extra_func()
            else:
                self.rpm_install_stack.set_visible_child_name("error_rpm_textview_page")
                self.error_rpm_textview.set_text(result)
        else:
            self.resolve_extra_func()

    def resolve_extra_func(self):
        dependencies, errors = self.redos_dnf.dependencies_local_check(self.rpm).split("|||")
        if "Операция отменена" in errors or "Operation aborted" in errors or "Операция прервана" in errors:
            dependencies = dependencies.split("\n")
            output = ""
            key = -1
            for i in range(0, len(dependencies)):
                if "=" in dependencies[i]:
                    key = i
                    break
            if key > -1:
                dependencies = dependencies[key + 1:]
            dependencies.remove('')
            for i in range(0, len(dependencies)):
                temp = dependencies[i].strip().split()
                print(temp)
                if len(temp) > 5:
                    dependencies[i] = f'\t{temp[0]}-{temp[2]}.{temp[1]} {temp[3]} {temp[4]}{temp[5]}'
                elif len(temp) == 5:
                    dependencies[i] = f'\t{temp[0]}-{temp[2]}.{temp[1]} {temp[3]} {temp[4]}'
                elif len(temp) == 2 and (temp[0] == 'Результат' or temp[0] == 'Transaction'):
                    dependencies[i] = ""
                else:
                    dependencies[i] = f'<b>{dependencies[i]}</b>'
                output += f"{dependencies[i]}\n"
            self.rpm_install_stack.set_visible_child_name("resolve_dep_page_rpm")
            self.resolve_dep_rpm_local.set_markup(output)
            self.resolve_dep_rpm_local.select_region(0, 0)
        elif "Отсутствуют действия для выполнения" in dependencies:
            self.rpm_install_stack.set_visible_child_name("error_rpm_textview_page")
            dependencies = dependencies.split("\n")
            output = ""
            key = -1
            for i in range(0, len(dependencies)):
                if "=" in dependencies[i]:
                    key = i
                    break
            if key > -1:
                dependencies = dependencies[key + 1:]
            for i in range(0, len(dependencies)):
                output += f"{dependencies[i]}\n"
            self.error_rpm_textview.set_text(output)
        else:
            self.rpm_install_stack.set_visible_child_name("error_rpm_textview_page")
            self.error_rpm_textview.set_text(errors)
    def close(self, button):
        self.utility_label.set_text(self.label_start_text)
        self.button_utility.set_sensitive(True)
        if os.path.exists("/tmp/yandex-browser.rpm"):
            os.remove("/tmp/yandex-browser.rpm")
        self.window.destroy()

    def on_accept_transaction_rpm_clicked(self, button):
        out = os.popen("systemctl is-active redoswelcome-dbus.service").readline().strip()
        if out == "inactive":
            the_dbus_starter.start()
        self.redos_dnf = bus.get("org.redoswelcome.pydbus.RedosWelcomeDNF", timeout=600)
        granted_access = self.redos_dnf.status_auth()
        if granted_access != "True":
            granted_access = self.redos_dnf.granteAuth(True)
        if granted_access == "rules granted":
            inputs = ()
            async_worker = AsyncWorker(
                operation=self.local_install,
                operation_inputs=inputs,
                operation_callback=self.on_result
            )
            async_worker.start()
        else:
            self.rpm_install_stack.set_visible_child_name("error_rpm_textview_page")
            self.error_rpm_textview.set_text("Operation not granted")
            the_dbus_starter.stop()


    def local_install(self,  *inputs):
        self.rpm_install_stack.set_visible_child_name("progress_page_rpm_local")
        result = self.redos_dnf.local_install(self.rpm)
        if result == "0":
            os.system("dnf clean all && dnf makecache")
        return result

    def on_result(self, worker, result, handler_data):
        result = worker.return_value(result)
        if result == "0":
            self.rpm_install_stack.set_visible_child_name("success_page_rpm_local")
            the_dbus_starter.stop()
        else:
            self.rpm_install_stack.set_visible_child_name("error_rpm_textview_page")
            self.error_rpm_textview.set_text(result)
            the_dbus_starter.stop()
